<?php

namespace Bixo\Note\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Note\Http\Requests\NoteRequest;
use Bixo\Note\Interfaces\NoteRepositoryInterface;
use Bixo\Note\Models\Note;

/**
 * Resource controller class for note.
 */
class NoteResourceController extends BaseController
{

    /**
     * Initialize note resource controller.
     *
     * @param type NoteRepositoryInterface $note
     *
     * @return null
     */
    public function __construct(NoteRepositoryInterface $note)
    {
        parent::__construct();
        $this->repository = $note;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Note\Repositories\Criteria\NoteResourceCriteria::class);
    }

    /**
     * Display a list of note.
     *
     * @return Response
     */
    public function index(NoteRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Note\Repositories\Presenter\NotePresenter::class)
                ->$function();
        }

        $notes = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('note::note.names'))
            ->view('note::note.index', true)
            ->data(compact('notes', 'view'))
            ->output();
    }

    /**
     * Display note.
     *
     * @param Request $request
     * @param Model   $note
     *
     * @return Response
     */
    public function show(NoteRequest $request, Note $note)
    {

        if ($note->exists) {
            $view = 'note::note.show';
        } else {
            $view = 'note::note.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('note::note.name'))
            ->data(compact('note'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new note.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NoteRequest $request)
    {

        $note = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('note::note.name')) 
            ->view('note::note.create', true) 
            ->data(compact('note'))
            ->output();
    }

    /**
     * Create new note.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NoteRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $note                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('note::note.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('note/note/' . $note->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/note/note'))
                ->redirect();
        }

    }

    /**
     * Show note for editing.
     *
     * @param Request $request
     * @param Model   $note
     *
     * @return Response
     */
    public function edit(NoteRequest $request, Note $note)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('note::note.name'))
            ->view('note::note.edit', true)
            ->data(compact('note'))
            ->output();
    }

    /**
     * Update the note.
     *
     * @param Request $request
     * @param Model   $note
     *
     * @return Response
     */
    public function update(NoteRequest $request, Note $note)
    {
        try {
            $attributes = $request->all();

            $note->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('note::note.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('note/note/' . $note->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('note/note/' . $note->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the note.
     *
     * @param Model   $note
     *
     * @return Response
     */
    public function destroy(NoteRequest $request, Note $note)
    {
        try {

            $note->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('note::note.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('note/note/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('note/note/' . $note->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple note.
     *
     * @param Model   $note
     *
     * @return Response
     */
    public function delete(NoteRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('note::note.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('note/note'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/note/note'))
                ->redirect();
        }

    }

    /**
     * Restore deleted notes.
     *
     * @param Model   $note
     *
     * @return Response
     */
    public function restore(NoteRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('note::note.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/note/note'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/note/note/'))
                ->redirect();
        }

    }

}
